package com.cr.models

import com.cr.data.DEFAULT_STR
import dji.v5.common.register.PackageProductCategory
import dji.v5.utils.inner.SDKConfig

/**
 * 操作系统：MAC系统
 * 创建者：王成
 * 创建日期：2023/3/7 16:38
 * 描述：封装和MSDK交互的逻辑，读取MSDK Info的相关信息
 */
class CrMSDKInfoModel {
    /**
     * 获取是否调试信息
     * @return Boolean
     */
    fun isDebug(): Boolean {
        return SDKConfig.getInstance().isDebug
    }

    /**
     * 获取分组信息
     * @return PackageProductCategory
     */
    fun getPackageProductCategory(): PackageProductCategory {
        return SDKConfig.getInstance().packageProductCategory
    }

    /**
     * 获取MSDK版本信息
     * @return String
     */
    fun getSDKVersion() : String {
        var config = SDKConfig.getInstance()
        return config.registrationSDKVersion?: DEFAULT_STR
    }

    /**
     * 获取编译版本信息
     * @return String
     */
    fun getBuildVersion() : String {
        return SDKConfig.getInstance().buildVersion
    }
}