package com.cr.models

/**
 * 操作系统：MAC系统
 * 创建者：王成
 * 创建日期：2023/4/11 09:14
 * 描述：设置子项模型
 */
class SetItemModel {
    var imageId = 0
    var title: String? = null
    var isArrow = false
    var isSwitch = false
    var action: SettingAction? = null
    var switchIsOn = false

    /**
     * 初始化
     * @param imgId Int 图标资源Id
     * @param title String 标题
     * @param isArrow Boolean 是否显示右侧箭头
     * @param isSwitch Boolean 是否显示开关
     * @param action SettingAction 动作
     * @param switchIsOn Boolean 开关是否开启
     * @constructor
     */
    constructor(
        imgId: Int,
        title: String,
        isArrow: Boolean,
        isSwitch: Boolean,
        action: SettingAction,
        switchIsOn: Boolean
    ) {
        this.imageId = imgId
        this.title = title
        this.isArrow = isArrow
        this.isSwitch = isSwitch
        this.action = action
        this.switchIsOn = switchIsOn
    }
}

/**
 * 操作类型
 */
enum class SettingAction {
    /**
     * 登录大疆账号
     */
    SA_DJI_LOGIN,

    /**
     * 下载数据
     */
    SA_DOWNLOAD_DATA,

    /**
     * 地图居中
     */
    SA_MAP_CENTER,

    /**
     * 设置IP和COM
     */
    SA_SET_IP_COM,

    /**
     * 查看历史航线
     */
    SA_OLD_AIRLINE,

    /**
     * 重新登录系统
     */
    SA_RE_LOGIN,

    /**
     * 删除地图缓存
     */
    SA_DEL_MAP_CACHE,

    /**
     * 查看大疆飞行记录
     */
    SA_SHOW_DJI_LOG,

    /**
     * 删除取证数据
     */
    SA_DEL_QZ_DATA,

    /**
     * 删除航线数据
     */
    SA_DEL_LINE_DATA,

    /**
     * 删除资源数据
     */
    SA_DEL_RESOURCE_DATA,

    /**
     * 上传断网航点
     */
    SA_UPLOAD_NO_UP_AIR_POINT,

    /**
     * 模拟器
     */
    SA_SIMULATOR,
}