package com.cr.pages

import android.app.Activity.RESULT_OK
import android.content.Intent
import android.os.Build
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import android.widget.TextView
import com.cr.common.CrPictureManager
import com.cr.common.CrFileManager
import com.cr.common.CrUnitManager
import com.cr.common.DataManager
import com.cr.cruav.CrApplication
import com.cr.cruav.R
import com.cr.data.CrConfig
import com.cr.data.CrUtil
import com.cr.event.EventMapCapture
import com.cr.map.CaseModel
import com.cr.models.CompletionModel
import com.cr.models.ICompletion
import com.cr.widget.CrImageBrowserWidget
import com.squareup.otto.Subscribe
import java.io.File

/**
 * 操作系统：MAC系统
 * 创建者：王成
 * 创建日期：2023/6/15 16:06
 * 描述：案件上传主页面
 */
class FragmentUploadCaseMain : CrNavigationFragment(), View.OnClickListener {
    /**
     * 操作接口
     */
    interface IOperationListener {
        // todo: 2023/6/20 编辑照片
        fun onEditImage(imagePath: String)

        // todo: 2023/8/18
        fun onUploadCase(joinCase: CaseModel)
    }

    companion object {
        const val OPEN_ALBUM = 3  // define: 2023/6/16 相册
    }

    private var lblCaseName: TextView? = null // define: 2023/6/15 案件名称
    private var lblCaseImageCount: TextView? = null // define: 2023/6/15 案件照片数量
    private var lblCaseCreateDate: TextView? = null // define: 2023/6/15 案件创建日期
    private var lblCaseMediaName: TextView? = null // define: 2023/6/15 案件媒体点名称
    private var lblCaseMediaSize: TextView? = null // define: 2023/6/15 案件媒体大小
    private var btnEdit: LinearLayout? = null // define: 2023/6/15 编辑按钮
    private var btnPhoto: LinearLayout? = null // define: 2023/6/15 相册按钮
    private var btnUpload: LinearLayout? = null // define: 2023/6/15 上传按钮
    private var imageBrowser: CrImageBrowserWidget? = null // define: 2023/6/16 图片浏览控件

    private var joinCase: CaseModel? = null // define: 2023/6/15 关联的案件点
    private var listener: IOperationListener? = null // define: 2023/6/20 监听
    private var cursorImageName: String? = null // define: 2023/6/20 照片名称

    /**
     * 初始化
     * @param inflater LayoutInflater
     * @param container ViewGroup?
     * @param savedInstanceState Bundle?
     * @return View?
     */
    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        mainView = inflater.inflate(R.layout.frag_case_upload_main, null)
        self = this
        // todo: 2023/6/15 挂载控件
        joinControls()
        // todo: 2023/6/15 显示
        if (this.joinCase != null) {
            crSetJoinCase(this.joinCase!!)
        }
        // todo: 2023/8/25 订阅事件
        CrApplication.getEventBus().register(this)
        return mainView
    }

    /**
     * 覆写挂载控件
     */
    override fun joinControls() {
        mainView?.let {
            lblCaseName = it.findViewById(R.id.case_name)
            lblCaseImageCount = it.findViewById(R.id.case_image_count)
            lblCaseCreateDate = it.findViewById(R.id.case_create_date)
            lblCaseMediaName = it.findViewById(R.id.case_media_name)
            lblCaseMediaSize = it.findViewById(R.id.case_media_size)

            btnEdit = it.findViewById(R.id.case_btn_edit)
            btnEdit?.setOnClickListener(this)

            btnPhoto = it.findViewById(R.id.case_btn_photo)
            btnPhoto?.setOnClickListener(this)

            btnUpload = it.findViewById(R.id.case_btn_upload)
            btnUpload?.setOnClickListener(this)

            imageBrowser = it.findViewById(R.id.case_image_browser)
            imageBrowser?.crSetListener(imageBrowserListener)
        }
    }

    // todo: 2023/6/17 滑动监听
    private val imageBrowserListener = object : CrImageBrowserWidget.iChangeListener {
        // todo: 2023/6/17 滑动监听
        override fun onChange(file: CrImageBrowserWidget.ImageFileModel?) {
            cursorImageName = file?.fileName
            lblCaseMediaName?.text = cursorImageName
            lblCaseMediaSize?.text = file?.fileSizeDescription
        }
    }

    /**
     * 覆写点击事件
     * @param view View
     */
    override fun onClick(view: View?) {
        when (view?.id) {
            // todo: 2023/6/15 编辑照片
            R.id.case_btn_edit -> {
                if (cursorImageName == null) {
                    showWarning("未选择需要编辑的照片！")
                } else {
                    if (listener != null) listener?.onEditImage(
                        String.format(
                            "%s%s",
                            CrUtil.IMAGE_PATH,
                            cursorImageName
                        )
                    )
                }
            }
            // todo: 2023/6/15 打开相册
            R.id.case_btn_photo -> {
                Intent(Intent.ACTION_PICK).apply {
                    type = "image/*"
                    startActivityForResult(this, OPEN_ALBUM)
                }
            }
            // todo: 2023/6/15 上传
            R.id.case_btn_upload -> {
                checkDataAndUpdateCase()
            }
        }
    }

    /**
     * 检查数据并长传案件
     */
    private fun checkDataAndUpdateCase() {
        // todo: 2023/6/20 查询数据库中数据
        DataManager.appQueryImages(this.joinCase!!.name!!, object : ICompletion<List<String>> {
            override fun onCompletion(completion: CompletionModel<List<String>>) {
                if (completion.isSuccess == true) {
                    // todo: 2023/8/25 进行判断
                    if (completion.result!!.size > 1) {
                        checkCaseCaptureData()
                    } else {
                        showWarning("不存在取证照片，无法上传！");
                    }
                }
            }
        })
    }

    /**
     * 检查案件截屏数据
     */
    private fun checkCaseCaptureData() {
        // todo: 2023/8/25 先调用地图 进行截图 截图之前进行判断 是否已经截取
        var isExists = true
        val names = CrFileManager.getCaptureNames(joinCase!!.name!!)
        for (name in names) {
            val filePath = "${CrUtil.IMAGE_PATH}${name}"
            isExists = isExists && CrFileManager.isExists(filePath)
        }
        if (isExists) {
            showConfirm("截图文件已存在，是否直接使用？", arrayOf("是","否"),object:ICompletion<String>{
                override fun onCompletion(completion: CompletionModel<String>) {
                    if(completion.isSuccess == true){
                        sendListener()
                    }else{
                        // todo: 2023/8/25 发送截屏事件
                        sendCaptureEvent()
                    }
                }
            })
        } else {
            // todo: 2023/8/25 发送截屏事件
            sendCaptureEvent()
        }
    }

    /**
     * 发送截屏事件
     */
    private fun sendCaptureEvent(){
        CrApplication.getEventBus().post(
            EventMapCapture(
                EventMapCapture.CaptureAction.CAPTURE_ACTION_START,
                joinCase!!.name
            )
        )
    }

    /**
     * 设置关联的案件点
     * @param joinCase CaseModel 案件点
     */
    fun crSetJoinCase(joinCase: CaseModel) {
        // todo: 2023/8/26 初始化图片浏览器 必须的
        imageBrowser?.crInit()
        this.joinCase = joinCase
        // todo: 2023/6/15 设置显示内容
        lblCaseName?.text = this.joinCase?.name
        lblCaseCreateDate?.text = this.joinCase?.date
        lblCaseImageCount?.text = String.format("取证照片[%s]张", this.joinCase?.imgArray?.size)
        lblCaseMediaName?.text = ""
        lblCaseMediaSize?.text = ""
        // todo: 2023/6/20 查询数据库中数据
        DataManager.appQueryImages(this.joinCase!!.name!!, object : ICompletion<List<String>> {
            override fun onCompletion(completion: CompletionModel<List<String>>) {
                if (completion.isSuccess == true) {
                    // todo: 2023/6/20 更新显示
                    lblCaseImageCount?.text = String.format("取证照片[%s]张", completion.result!!.size)
                    // todo: 2023/6/20 追加到照片浏览器中
                    for (imgName in completion.result!!) {
                        var imgPath = String.format("%s%s", CrUtil.IMAGE_PATH, imgName)
                        imageBrowser?.crAppendImage(imgPath)
                    }
                }
            }
        })
    }

    /**
     * 覆盖外部调用返回接受并处理
     * @param requestCode Int
     * @param resultCode Int
     * @param data Intent?
     */
    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == OPEN_ALBUM && resultCode == RESULT_OK) {
            var imagePath: String? = null
            imagePath = if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
                CrPictureManager.handleImageAfterKitKat(context!!, data!!)
            } else {
                CrPictureManager.handleImageBeforeKitKat(context!!, data!!)
            }
            if (imagePath != null) {
                // todo: 2023/6/16 转存文件
                var newFilePath =
                    String.format("%sUAV%s.jpg", CrUtil.IMAGE_PATH, CrUnitManager.toSystemDate())
                var res = CrFileManager.copyFile(imagePath, newFilePath)
                if (res) {
                    // todo: 2023/6/19 先存储到数据库
                    var fileName = CrFileManager.getFileName(newFilePath)
                    var model = CaseModel()
                    model.name = this.joinCase!!.name
                    model.imgName = fileName
                    DataManager.appAppendImages(model, object : ICompletion<String> {
                        override fun onCompletion(completion: CompletionModel<String>) {
                            if (completion.isSuccess == true) {
                                // todo: 2023/6/20 添加到照片容器
                                imageBrowser?.crAppendImage(newFilePath)
                                // todo: 2023/6/20 刷新照片熟练
                                DataManager.appQueryImages(
                                    joinCase!!.name!!,
                                    object : ICompletion<List<String>> {
                                        override fun onCompletion(completion: CompletionModel<List<String>>) {
                                            if (completion.isSuccess == true) {
                                                // todo: 2023/6/20 更新显示
                                                lblCaseImageCount?.text = String.format(
                                                    "取证照片[%s]张",
                                                    completion.result!!.size
                                                )
                                            }
                                        }
                                    })
                            } else {
                                showError(completion.result!!)
                            }
                        }
                    })
                } else {
                    showError("文件转存失败！")
                }
            } else {
                showError("选择文件异常！")
            }
        }
    }

    /**
     * 发送监听
     */
    private fun sendListener(){
        // todo: 2023/8/25 对关联案件进行设置
        joinCase?.let { itCase ->
            // todo: 2023/8/25 清理数组
            if(itCase.imgArray == null) itCase.imgArray = mutableListOf()
            else itCase.imgArray?.clear()
            // todo: 2023/8/25 追加图片
            DataManager.appQueryImages(this.joinCase!!.name!!, object : ICompletion<List<String>> {
                override fun onCompletion(completion: CompletionModel<List<String>>) {
                    if (completion.isSuccess == true) {
                        for(imgName in completion.result!!){
                            itCase.imgArray?.add(imgName)
                        }
                        // todo: 2023/8/25 追截截图
                        var captureNames = CrFileManager.getCaptureNames(itCase.name!!)
                        for(imgName in captureNames){
                            itCase.imgArray?.add(imgName)
                        }
                        // todo: 2023/8/25 回调打开窗口
                        listener?.let { it.onUploadCase(joinCase!!) }
                    }
                }
            })
        }
    }

    /**
     * 订阅截屏事件
     */
    @Subscribe
    fun onCapture(event: EventMapCapture) {
        if (event.action == EventMapCapture.CaptureAction.CAPTURE_ACTION_COMPLETE) {
            // todo: 2023/8/26 发送监听 打开页面
            sendListener()
        }
    }

    // todo: 2023/6/20 设置监听
    fun crSetListener(listener: IOperationListener) {
        this.listener = listener
    }

    // todo: 2023/8/25 生命周期
    override fun onDestroy() {
        super.onDestroy()
        // todo: 2023/8/25 取消订阅事件
        CrApplication.getEventBus().unregister(this)
    }
}